function xUL_f2d_xM_ttau0(dxdt_fnc,xgrd,f0,t_op,tau_op,ido_bwd)
% xUL_f2d_xM_ttau0: compute x and M
%   input
%      dxdt_fnc:    flow model
%      xgrd:        tracer ic
%      f0:          flow ic
%      t_op:        t0
%      tau_op:      tau
%      ido_bwd:     do or do not backward computation
%   output into dfnm_op.x,M,f
%
% Written by Kayo Ide (University of Maryland)
% 2015 August

%% global
% system input (passed on)
global Nd
global ido idont
% 
global ixMf_strt ixMf_end
global ido_xMf
% 
global dfnm_op

%% set up 
cpu0=cputime;

%% initialization
% xMf0: x & M initialization
%- x on spatial grid
[Xgrd1,Xgrd2]=meshgrid(linspace(xgrd(1).mn,xgrd(1).mx,xgrd(1).n),...
    linspace(xgrd(2).mn,xgrd(2).mx,xgrd(2).n));
%- M on spatial grid
Mgrd=zeros(size(Xgrd1));
Np=numel(Mgrd);
%- xMf0
xM0=[Xgrd1(:); Xgrd2(:); Mgrd(:)];
xMf0=[Xgrd1(:); Xgrd2(:); Mgrd(:); f0(:)];

%% compute analytic flow trajectory for IC to compute t_tau0
ido_xMf=idont;
[tk,xMfk]=ode45(dxdt_fnc,t_op,[xM0; f0(:)]');
fk=xMfk(:,ixMf_strt.f:end)'; 
    
%% t: temporal resolution
Kt=size(t_op,2);
% output: xfwd/bwd & Mfwd/bwd 
%         ffwd/bwd
xfwd=zeros(Np,Kt,Nd);  Mfwd=zeros(Np,Kt); 
ffwd=fk;
if(ido_bwd-ido==0); xbwd=xfwd; Mbwd=Mfwd; fbwd=ffwd;
else; xbwd=0; Mbwd=0; fbwd=0;
end

%% Computation of M
ido_xMf=ido;
% forward
fprintf('\n        M up to t_op(1:%d)=[%2.2f:%2.2f:%2.2f] at tau0=%2.2f',...
    Kt,t_op(1),t_op(2)-t_op(1),t_op(end),tau_op(1));
k=1; xMf0k=[xMf0]';
for k=1:Kt;
    % initilaiation
    fprintf('\n               t_op=%2.2f',t_op(k));
    % forward
    fprintf('\n               ... forward');
    [tauk,xMfk]=ode45(dxdt_fnc,t_op(k)+[0,tau_op(k)/2,tau_op(k)],[xM0; fk(:,k)]'); 
    for nd=1:Nd
        xfwd(:,k,nd)=xMfk(end,[ixMf_strt.x(nd):ixMf_end.x(nd)])';
    end
    Mfwd(:,k)=xMfk(end,[ixMf_strt.M:ixMf_end.M])';
    ffwd(:,k)=fk(:,k);
    % backward
    if(ido_bwd-ido==0); 
        fprintf('\n               ... backward');
        [tauk,xMfk]=ode45(dxdt_fnc,t_op(k)-[0,tau_op(k)/2,tau_op(k)],[xM0; fk(:,k)]'); 
        for nd=1:Nd
            xbwd(:,k,nd)=xMfk(end,[ixMf_strt.x(nd):ixMf_end.x(nd)])';
        end
        Mbwd(:,k)=xMfk(end,[ixMf_strt.M:ixMf_end.M])';
        fbwd(:,k)=fk(:,k);
    end
end

%% output
xgrd_op=xgrd;
save(dfnm_op.x,'-mat','t_op','tau_op','xgrd_op','xfwd','xbwd')
save(dfnm_op.M,'-mat','t_op','tau_op','xgrd_op','Mfwd','Mbwd')
save(dfnm_op.f,'-mat','t_op','tau_op','xgrd_op','ffwd','fbwd')

fprintf('\n        cpu for xM=%10.2f',cputime-cpu0);

return